#!/usr/local/bin/perl

# Script to change ppm.out file atom naming. Use if you have
# only one atom name for methyls (e.g. HD11 by itself instead of
# HD11,HD12 and HD13).
#
# Use as: nmrviewppm.atomname.pl <input ppm file> <input sequence file> <output ppm file>

&initialize;

&readppm;

&getsequence;

&changeppm;

sub changeppm {

	my ($atom_count,$resnum) =0;
	my ($atname)="";

	# First loop to check number of occurences.

	for ($atom_count=0;$atom_count<=$#atom;$atom_count++) {
		# Only change for protons
		if ($atom_type[$atom_count] eq 'H') {
			($resnum,$atname)=split(/\./,$atom[$atom_count]);
			$atid=substr($atname,0,length($atname)-1);
			if ($atid =~ /^H\w+/) {
				$atid=$res[$resnum]." ".$atid;
				if ($atid =~ /$methyls/) {
					$methyllist{$resnum.$atid}++;
				}
				elsif ($atid =~ /$methylenes/) {
					$methyleneslist{$resnum.$atid}++;
					#print $atid." ".$methyleneslist{$resnum.$atid}."\n";
				}
			}
		}
	}
	
	# Second loop to figure out if need to add an atom and print it.
	
	for ($atom_count=0;$atom_count<=$#atom;$atom_count++) {
		($resnum,$atname)=split(/\./,$atom[$atom_count]);
		# Only change for protons
		if ($atom_type[$atom_count] eq 'H') {
			$atid=substr($atname,0,length($atname)-1);
			if ($atid =~ /^H\w+/) {
				$atid=$res[$resnum]." ".$atid;
				if ($atid =~ /$methyls/ && $methyllist{$resnum.$atid} != 3) {
					for ($i=1;$i<=3;$i++) {
						&printppm($resnum,substr($atname,0,length($atname)-1).$i,$ppm[$atom_count],$scode[$atom_count]);
						print ("Added atoms for residue $resnum, $atid\n");
					}
				}
				elsif ($atid =~ /$methylenes/ && $methyleneslist{$resnum.$atid} != 2) {
					for ($i=1;$i<=2;$i++) {
						&printppm($resnum,substr($atname,0,length($atname)-1).$i,$ppm[$atom_count],$scode[$atom_count]);
						print ("Added atoms for residue $resnum, $atid\n");
					}
				}
				else {
					&printppm($resnum,$atname,$ppm[$atom_count],$scode[$atom_count]);
				}
			}
			else {
				&printppm($resnum,$atname,$ppm[$atom_count],$scode[$atom_count]);
			}
		}
		else {
			&printppm($resnum,$atname,$ppm[$atom_count],$scode[$atom_count]);
		}
	}
}

sub printppm {
	my ($resnum,$atom,$ppm,$scode)=@_;
	printf OUTPUT ("%3d.%-6s %7.3f %d\n",$resnum,$atom,$ppm,$scode);	
}

sub getsequence {

	my $resnum = 1;

	while (defined ($_=<SEQUENCE>)) {
		next if (!/[A-Z|a-z]+/);
		chomp;
		$res[$resnum++]=uc($_);
	}
}


sub initialize {

	# DEFAULTS

	require "/ozzy/perlscripts/aria.globals";

	# Open input

	$status=open(PPMINPUT,$ARGV[0]);
	if ($status==0) {die "Problems reading $ARGV[0]: $!"};
	
	$status=open(SEQUENCE,$ARGV[1]);
	if ($status==0) {die "Problems reading $ARGV[1]: $!"};
	
	&openoutputfile($ARGV[2],OUTPUT,"");


}

sub openoutputfile {

	my ($file,$handle,$text)=@_;

	# Open output file with header line

	open ($handle,">$file") || die "Unable to open file $file: $!";

	print $handle $text;
}

sub readppm {

	my ($atom_all,$ppm,$atom_count)=0;

	while (defined ($_=<PPMINPUT>)) {
		next if (!/\d+\.\w+/);
		($atom_all,$ppm,$scode)=split(' ');
		$atom_type[$atom_count]=substr($atom_all,index($atom_all,'.')+1,1);
		$atom[$atom_count]=$atom_all;
		$ppm[$atom_count]=$ppm;
		$scode[$atom_count]=$scode;
		$atom_count++;
	}
			
	close PPMINPUT;
}
